@@ -29,7 +29,7 @@
 
 #include "machine.h"
 #include "machines_periph.h"
-#include "memory.h"
+#include "memory_pages.h"
 #include "periph.h"
 #include "peripherals/disk/beta.h"
 #include "settings.h"
