@@ -1,11 +1,6 @@
 /* menu.c: general menu callbacks
-   Copyright (c) 2004-2013 Philip Kendall
-   Copyright (c) 2013 Alex Badea
-   Copyright (c) 2014-2015 Sergio Baldoví
-   Copyright (c) 2015 Stuart Brady
-   Copyright (c) 2015-2017 Gergely Szasz
-   Copyright (c) 2015 Stefano Bodrato
-   Copyright (c) 2017 Fredrick Meunier
+   Copyright (c) 2004-2017 Philip Kendall, Alex Badea, Sergio Baldoví,
+     Stuart Brady, Gergely Szasz, Stefano Bodrato, Fredrick Meunier
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,13 +38,16 @@
 #include "peripherals/disk/opus.h"
 #include "peripherals/disk/plusd.h"
 #include "peripherals/ide/divide.h"
+#include "peripherals/ide/divmmc.h"
 #include "peripherals/ide/simpleide.h"
 #include "peripherals/ide/zxatasp.h"
 #include "peripherals/ide/zxcf.h"
+#include "peripherals/ide/zxmmc.h"
 #include "peripherals/if1.h"
 #include "peripherals/if2.h"
 #include "peripherals/joystick.h"
 #include "peripherals/multiface.h"
+#include "peripherals/scld.h"
 #include "profile.h"
 #include "psg.h"
 #include "rzx.h"
@@ -208,7 +206,7 @@ MENU_CALLBACK( menu_file_aylogging_stop )
   ui_menu_activate( UI_MENU_ITEM_AY_LOGGING, 0 );
 }
 
-MENU_CALLBACK( menu_file_openscrscreenshot )
+MENU_CALLBACK( menu_file_screenshot_openscrscreenshot )
 {
   char *filename;
 
@@ -224,6 +222,22 @@ MENU_CALLBACK( menu_file_openscrscreenshot )
   fuse_emulation_unpause();
 }
 
+MENU_CALLBACK( menu_file_screenshot_openmltscreenshot )
+{
+  char *filename;
+
+  fuse_emulation_pause();
+
+  filename = ui_get_open_filename( "Fuse - Open MLT Screenshot" );
+  if( !filename ) { fuse_emulation_unpause(); return; }
+
+  screenshot_mlt_read( filename );
+
+  libspectrum_free( filename );
+
+  fuse_emulation_unpause();
+}
+
 MENU_CALLBACK( menu_file_movie_stop )
 {
   ui_widget_finish();
@@ -629,6 +643,8 @@ MENU_CALLBACK_WITH_ACTION( menu_media_ide_insert )
   case 5: zxcf_insert( filename ); break;
   case 6: divide_insert( filename, LIBSPECTRUM_IDE_MASTER ); break;
   case 7: divide_insert( filename, LIBSPECTRUM_IDE_SLAVE  ); break;
+  case 8: divmmc_insert( filename ); break;
+  case 9: zxmmc_insert( filename ); break;
   }
 
   libspectrum_free( filename );
@@ -648,6 +664,8 @@ MENU_CALLBACK_WITH_ACTION( menu_media_ide_commit )
   case 5: zxcf_commit(); break;
   case 6: divide_commit( LIBSPECTRUM_IDE_MASTER ); break;
   case 7: divide_commit( LIBSPECTRUM_IDE_SLAVE  ); break;
+  case 8: divmmc_commit(); break;
+  case 9: zxmmc_commit(); break;
   }
 
   fuse_emulation_unpause();
@@ -667,6 +685,7 @@ MENU_CALLBACK_WITH_ACTION( menu_media_ide_eject )
   case 5: zxcf_eject(); break;
   case 6: divide_eject( LIBSPECTRUM_IDE_MASTER ); break;
   case 7: divide_eject( LIBSPECTRUM_IDE_SLAVE  ); break;
+  case 8: divmmc_eject(); break;
   }
 
   fuse_emulation_unpause();
@@ -692,13 +711,6 @@ MENU_CALLBACK( menu_media_ide_zxcf_upload )
   ui_widget_finish();
 }
 
-MENU_CALLBACK( menu_media_ide_divide_writeprotect )
-{
-  settings_current.divide_wp = !settings_current.divide_wp;
-  divide_refresh_page_state();
-  ui_widget_finish();
-}
-
 MENU_CALLBACK( menu_file_savesnapshot )
 {
   char *filename;
@@ -717,7 +729,7 @@ MENU_CALLBACK( menu_file_savesnapshot )
   fuse_emulation_unpause();
 }
 
-MENU_CALLBACK( menu_file_savescreenasscr )
+MENU_CALLBACK( menu_file_screenshot_savescreenasscr )
 {
   char *filename;
 
@@ -735,9 +747,34 @@ MENU_CALLBACK( menu_file_savescreenasscr )
   fuse_emulation_unpause();
 }
 
+MENU_CALLBACK( menu_file_screenshot_savescreenasmlt )
+{
+  char *filename;
+
+  ui_widget_finish();
+
+  fuse_emulation_pause();
+
+  if( machine_current->timex && scld_last_dec.name.hires ) {
+    ui_error( UI_ERROR_ERROR,
+              "MLT format not supported for Timex hi-res screen mode" );
+    fuse_emulation_unpause();
+    return;
+  }
+
+  filename = ui_get_save_filename( "Fuse - Save Screenshot as MLT" );
+  if( !filename ) { fuse_emulation_unpause(); return; }
+
+  screenshot_mlt_write( filename );
+
+  libspectrum_free( filename );
+
+  fuse_emulation_unpause();
+}
+
 #ifdef USE_LIBPNG
 
-MENU_CALLBACK( menu_file_savescreenaspng )
+MENU_CALLBACK( menu_file_screenshot_savescreenaspng )
 {
   scaler_type scaler;
   char *filename;
@@ -1018,6 +1055,16 @@ menu_check_media_changed( void )
     if( confirm ) return 1;
   }
 
+  if( settings_current.divmmc_file ) {
+    confirm = divmmc_eject();
+    if( confirm ) return 1;
+  }
+
+  if( settings_current.zxmmc_file ) {
+    confirm = zxmmc_eject();
+    if( confirm ) return 1;
+  }
+
   return 0;
 }
 
