@@ -1,8 +1,6 @@
 /* beta.c: Routines for handling the Beta disk interface
    Copyright (c) 2004-2016 Stuart Brady, Philip Kendall, Gergely Szasz
 
-   $Id: beta.c 5677 2016-07-09 13:58:02Z fredm $
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -392,9 +390,17 @@ beta_disk_insert( beta_drive_number which, const char *filename,
 static int
 ui_drive_autoload( void )
 {
-  PC = 0;
-  machine_current->ram.last_byte |= 0x10;   /* Select ROM 1 */
-  beta_page();
+  /* Clear AY registers (and more) from current machine */
+  machine_reset(1);
+
+  if( ( machine_current->capabilities &
+        LIBSPECTRUM_MACHINE_CAPABILITY_128_MEMORY ) ||
+      !settings_current.beta128_48boot ) {
+    PC = 0;
+    machine_current->ram.last_byte |= 0x10;   /* Select ROM 1 */
+    beta_page();
+  }
+
   return 0;
 }
 
