@@ -1,8 +1,6 @@
 /* disk.h: Routines for handling disk images
    Copyright (c) 2007-2015 Gergely Szasz
 
-   $Id: disk.h 5498 2016-05-19 21:52:49Z sbaldovi $
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -26,8 +24,6 @@
 #ifndef FUSE_DISK_H
 #define FUSE_DISK_H
 
-#include <config.h>
-
 static const unsigned int DISK_FLAG_NONE = 0x00;
 static const unsigned int DISK_FLAG_PLUS3_CPC = 0x01;	/* try to fix some CPC issue */
 static const unsigned int DISK_FLAG_OPEN_DS = 0x02;	/* try to open the other side too */
@@ -130,6 +126,14 @@ TRACK_LEN TYPE TRACK......DATA CLOCK..MARKS MF..MARKS WEAK..MARKS
 #define DISK_SET_TRACK( d, head, cyl ) \
    DISK_SET_TRACK_IDX( (d), (d)->sides * cyl + head )
 
+typedef struct disk_position_context_t {
+  libspectrum_byte *track;   /* current track data bytes */
+  libspectrum_byte *clocks;  /* clock marks bits */
+  libspectrum_byte *fm;      /* FM/MFM marks bits */
+  libspectrum_byte *weak;    /* weak marks bits/weak data */
+  int i;                     /* index for track and clocks */
+} disk_position_context_t;
+
 const char *disk_strerror( int error );
 /* create an unformatted disk sides -> (1/2) cylinders -> track/side,
    dens -> 'density' related to unformatted length of a track (SD = 3125,
