@@ -4,8 +4,6 @@
    Copyright (c) 2009-2016 Patrik Persson, Philip Kendall
    Copyright (c) 2015 Stuart Brady
    
-   $Id: speccyboot.c 5677 2016-07-09 13:58:02Z fredm $
-   
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -32,7 +30,7 @@
 #include "debugger/debugger.h"
 #include "infrastructure/startup_manager.h"
 #include "machine.h"
-#include "memory.h"
+#include "memory_pages.h"
 #include "nic/enc28j60.h"
 #include "module.h"
 #include "periph.h"
