@@ -156,7 +156,7 @@ create_dialog( void )
   /* Create Add button for custom pokes */
   static const gtkstock_button
     add  = { "_Add", G_CALLBACK( pokemem_add_custom_poke ), NULL, NULL,
-             0, 0, 0, 0 };
+             0, 0, 0, 0, GTK_RESPONSE_NONE };
   gtkstock_create_button( GTK_WIDGET( hbox ), accel_group, &add );
 
   label = gtk_label_new( "Choose active POKES:" );
@@ -182,7 +182,6 @@ create_dialog( void )
                              (gpointer) &dialog,
                              G_CALLBACK( pokemem_close ),
                              G_CALLBACK( pokemem_close ) );
-  gtk_accel_group_disconnect_key( accel_group, GDK_KEY_Return, 0 );
 
   /* Users shouldn't be able to resize this window */
   gtk_window_set_resizable( GTK_WINDOW( dialog ), FALSE );
