@@ -234,6 +234,10 @@ typedef enum ui_menu_item {
   UI_MENU_ITEM_MEDIA_IDE_DIVIDE,
   UI_MENU_ITEM_MEDIA_IDE_DIVIDE_MASTER_EJECT,
   UI_MENU_ITEM_MEDIA_IDE_DIVIDE_SLAVE_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE_DIVMMC,
+  UI_MENU_ITEM_MEDIA_IDE_DIVMMC_EJECT,
+  UI_MENU_ITEM_MEDIA_IDE_ZXMMC,
+  UI_MENU_ITEM_MEDIA_IDE_ZXMMC_EJECT,
   UI_MENU_ITEM_RECORDING,
   UI_MENU_ITEM_RECORDING_ROLLBACK,
   UI_MENU_ITEM_AY_LOGGING,
