@@ -2,8 +2,6 @@
    Copyright (c) 2002-2005 Philip Kendall
    Copyright (c) 2015 Stuart Brady
 
-   $Id: text.c 5434 2016-05-01 04:22:45Z fredm $
-
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -35,6 +33,7 @@ char *widget_text_text = NULL;	/* What we return the text in */
 
 static const char *title;	/* The window title */
 static widget_text_input_allow allow;
+static unsigned int max_length;
 
 #define WIDGET_TEXT_LENGTH 64
 static char text[WIDGET_TEXT_LENGTH];	/* The current entry text */
@@ -51,6 +50,7 @@ widget_text_draw( void *data )
   if( data ) {
     title = text_data->title;
     allow = text_data->allow;
+    max_length = text_data->max_length;
     snprintf( text, sizeof( text ), "%s", text_data->text );
   }
 
@@ -154,7 +154,7 @@ append_character( char c )
 {
   size_t length = strlen( text );
 
-  if( length < 23 ) {
+  if( length < WIDGET_TEXT_LENGTH - 1 && length < max_length ) {
     text[ length ] = c; text[ length + 1 ] = '\0';
   }
 }
