@@ -43,9 +43,11 @@
 #include "memory_pages.h"
 #include "peripherals/dck.h"
 #include "peripherals/ide/divide.h"
+#include "peripherals/ide/divmmc.h"
 #include "peripherals/ide/simpleide.h"
 #include "peripherals/ide/zxatasp.h"
 #include "peripherals/ide/zxcf.h"
+#include "peripherals/ide/zxmmc.h"
 #include "peripherals/if1.h"
 #include "peripherals/if2.h"
 #include "pokefinder/pokemem.h"
@@ -194,6 +196,8 @@ utils_open_file( const char *filename, int autoload,
     if( !settings_current.simpleide_active &&
 	!settings_current.zxatasp_active   &&
 	!settings_current.divide_enabled   &&
+	!settings_current.divmmc_enabled   &&
+	!settings_current.zxmmc_enabled    &&
 	!settings_current.zxcf_active         ) {
       settings_current.zxcf_active = 1;
       periph_update();
@@ -205,8 +209,12 @@ utils_open_file( const char *filename, int autoload,
       error = zxatasp_insert( filename, LIBSPECTRUM_IDE_MASTER );
     } else if( settings_current.simpleide_active ) {
       error = simpleide_insert( filename, LIBSPECTRUM_IDE_MASTER );
-    } else {
+    } else if( settings_current.divide_enabled ) {
       error = divide_insert( filename, LIBSPECTRUM_IDE_MASTER );
+    } else if( settings_current.zxmmc_enabled ) {
+      error = zxmmc_insert( filename );
+    } else {
+      error = divmmc_insert( filename );
     }
     if( error ) return error;
     
