@@ -1,7 +1,6 @@
 #!/usr/bin/perl -w
 
 # z80.pl: generate C code for Z80 opcodes
-# $Id: z80.pl 5434 2016-05-01 04:22:45Z fredm $
 
 # Copyright (c) 1999-2015 Philip Kendall
 
@@ -61,7 +60,8 @@ sub arithmetic_logical ($$$) {
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); PC++;
-	bytetemp = readbyte( REGISTER + (libspectrum_signed_byte)offset );
+	z80.memptr.w = REGISTER + (libspectrum_signed_byte)offset;
+	bytetemp = readbyte( z80.memptr.w );
 	$opcode(bytetemp);
       }
 CODE
@@ -104,6 +104,11 @@ sub call_jp ($$$) {
 
     my( $opcode, $condition, $offset ) = @_;
 
+    print << "CALL";
+      z80.memptr.b.l = readbyte(PC++);
+      z80.memptr.b.h = readbyte(PC);
+CALL
+
     if( not defined $offset ) {
 	print "      $opcode();\n";
     } else {
@@ -117,7 +122,7 @@ sub call_jp ($$$) {
       if( $condition_string ) {
 	$opcode();
       } else {
-	contend_read( PC, 3 ); contend_read( PC + 1, 3 ); PC += 2;
+        PC++;
       }
 CALL
     }
@@ -144,6 +149,7 @@ sub cpi_cpd ($) {
 	  ( bytetemp & FLAG_S );
 	if(F & FLAG_H) bytetemp--;
 	F |= ( bytetemp & FLAG_3 ) | ( (bytetemp&0x02) ? FLAG_5 : 0 );
+	z80.memptr.w$modifier;
       }
 CODE
 }
@@ -174,6 +180,9 @@ sub cpir_cpdr ($) {
 	  contend_read_no_mreq( HL, 1 ); contend_read_no_mreq( HL, 1 );
 	  contend_read_no_mreq( HL, 1 );
 	  PC-=2;
+	  z80.memptr.w = PC+1;
+	} else {
+	  z80.memptr.w$modifier;
 	}
 	HL$modifier;
       }
@@ -207,16 +216,15 @@ CODE
 	print << "CODE";
       {
 	libspectrum_byte offset, bytetemp;
-	libspectrum_word wordtemp;
 	offset = readbyte( PC );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); PC++;
-	wordtemp = REGISTER + (libspectrum_signed_byte)offset;
-	bytetemp = readbyte( wordtemp );
-	contend_read_no_mreq( wordtemp, 1 );
+	z80.memptr.w = REGISTER + (libspectrum_signed_byte)offset;
+	bytetemp = readbyte( z80.memptr.w );
+	contend_read_no_mreq( z80.memptr.w, 1 );
 	$opcode(bytetemp);
-	writebyte(wordtemp,bytetemp);
+	writebyte(z80.memptr.w,bytetemp);
       }
 CODE
     }
@@ -237,6 +245,7 @@ sub ini_ind ($) {
 	initemp = readport( BC );
 	writebyte( HL, initemp );
 
+	z80.memptr.w=BC $modifier 1;
         B--; HL$modifier$modifier;
         initemp2 = initemp + C $modifier 1;
 	F = ( initemp & 0x80 ? FLAG_N : 0 ) |
@@ -261,6 +270,7 @@ sub inir_indr ($) {
 	initemp = readport( BC );
 	writebyte( HL, initemp );
 
+	z80.memptr.w=BC $modifier 1;
 	B--;
         initemp2 = initemp + C $modifier 1;
 	F = ( initemp & 0x80 ? FLAG_N : 0 ) |
@@ -320,6 +330,7 @@ sub ldir_lddr ($) {
 	  contend_write_no_mreq( DE, 1 ); contend_write_no_mreq( DE, 1 );
 	  contend_write_no_mreq( DE, 1 );
 	  PC-=2;
+	  z80.memptr.w = PC+1;
 	}
         HL$modifier; DE$modifier;
       }
@@ -330,7 +341,7 @@ sub otir_otdr ($) {
 
     my( $opcode ) = @_;
 
-    my $modifier = ( $opcode eq 'OTIR' ? '++' : '--' );
+    my $modifier = ( $opcode eq 'OTIR' ? '+' : '-' );
 
     print << "CODE";
       {
@@ -339,9 +350,10 @@ sub otir_otdr ($) {
 	contend_read_no_mreq( IR, 1 );
 	outitemp = readbyte( HL );
 	B--;	/* This does happen first, despite what the specs say */
+	z80.memptr.w = BC $modifier 1;
 	writeport(BC,outitemp);
 
-	HL$modifier;
+	HL$modifier$modifier;
         outitemp2 = outitemp + L;
 	F = ( outitemp & 0x80 ? FLAG_N : 0 ) |
             ( ( outitemp2 < outitemp ) ? FLAG_H | FLAG_C : 0 ) |
@@ -362,7 +374,7 @@ sub outi_outd ($) {
 
     my( $opcode ) = @_;
 
-    my $modifier = ( $opcode eq 'OUTI' ? '++' : '--' );
+    my $modifier = ( $opcode eq 'OUTI' ? '+' : '-' );
 
     print << "CODE";
       {
@@ -371,9 +383,10 @@ sub outi_outd ($) {
 	contend_read_no_mreq( IR, 1 );
 	outitemp = readbyte( HL );
 	B--;	/* This does happen first, despite what the specs say */
+	z80.memptr.w = BC $modifier 1;
 	writeport(BC,outitemp);
 
-	HL$modifier;
+	HL$modifier$modifier;
         outitemp2 = outitemp + L;
 	F = ( outitemp & 0x80 ? FLAG_N : 0 ) |
             ( ( outitemp2 < outitemp ) ? FLAG_H | FLAG_C : 0 ) |
@@ -430,9 +443,9 @@ CODE
 	print << "CODE";
       {
 	libspectrum_byte bytetemp;
-	bytetemp = readbyte( tempaddr );
-	contend_read_no_mreq( tempaddr, 1 );
-	writebyte( tempaddr, bytetemp $operator $hex_mask );
+	bytetemp = readbyte( z80.memptr.w );
+	contend_read_no_mreq( z80.memptr.w, 1 );
+	writebyte( z80.memptr.w, bytetemp $operator $hex_mask );
       }
 CODE
     }
@@ -456,10 +469,10 @@ CODE
     } elsif( $register eq '(REGISTER+dd)' ) {
 	print << "CODE";
       {
-	libspectrum_byte bytetemp = readbyte(tempaddr);
-	contend_read_no_mreq( tempaddr, 1 );
+	libspectrum_byte bytetemp = readbyte(z80.memptr.w);
+	contend_read_no_mreq( z80.memptr.w, 1 );
 	$opcode(bytetemp);
-	writebyte(tempaddr,bytetemp);
+	writebyte(z80.memptr.w,bytetemp);
       }
 CODE
     }
@@ -482,9 +495,9 @@ sub opcode_BIT (@) {
     } elsif( $register eq '(REGISTER+dd)' ) {
 	print << "BIT";
       {
-	libspectrum_byte bytetemp = readbyte( tempaddr );
-	contend_read_no_mreq( tempaddr, 1 );
-	BIT_I( $bit, bytetemp, tempaddr );
+	libspectrum_byte bytetemp = readbyte( z80.memptr.w );
+	contend_read_no_mreq( z80.memptr.w, 1 );
+	BIT_MEMPTR( $bit, bytetemp );
       }
 BIT
     } else {
@@ -492,7 +505,7 @@ BIT
       {
 	libspectrum_byte bytetemp = readbyte( HL );
 	contend_read_no_mreq( HL, 1 );
-	BIT( $bit, bytetemp );
+	BIT_MEMPTR( $bit, bytetemp );
       }
 BIT
     }
@@ -554,8 +567,8 @@ sub opcode_DJNZ (@) {
 	JR();
       } else {
 	contend_read( PC, 3 );
+        PC++;
       }
-      PC++;
 DJNZ
 }
 
@@ -604,7 +617,8 @@ EX
 	writebyte( SP + 1, $high );
 	writebyte( SP,     $low  );
 	contend_write_no_mreq( SP, 1 ); contend_write_no_mreq( SP, 1 );
-	$low=bytetempl; $high=bytetemph;
+	$low=z80.memptr.b.l=bytetempl;
+	$high=z80.memptr.b.h=bytetemph;
       }
 EX
     } elsif( $arg1 eq 'DE' and $arg2 eq 'HL' ) {
@@ -642,10 +656,12 @@ sub opcode_IN (@) {
 
     if( $register eq 'A' and $port eq '(nn)' ) {
 	print << "IN";
-      { 
+      {
 	libspectrum_word intemp;
 	intemp = readbyte( PC++ ) + ( A << 8 );
         A=readport( intemp );
+	/* TODO: is this correct if (nn) was 0xff? */
+	z80.memptr.w = intemp + 1;
       }
 IN
     } elsif( $register eq 'F' and $port eq '(C)' ) {
@@ -704,11 +720,10 @@ sub opcode_JR (@) {
         JR();
       } else {
         contend_read( PC, 3 );
+	PC++;
       }
 JR
     }
-
-    print "      PC++;\n";
 }
 
 sub opcode_LD (@) {
@@ -745,16 +760,20 @@ LD
 	    print "      $dest = readbyte( PC++ );\n";
 	} elsif( $src =~ /^\(..\)$/ ) {
 	    my $register = substr $src, 1, 2;
+	    if( $register eq 'BC' or $register eq 'DE') {
+	        print << "LD";
+      z80.memptr.w=$register+1;
+LD
+            }
 	    print << "LD";
       $dest=readbyte($register);
 LD
         } elsif( $src eq '(nnnn)' ) {
 	    print << "LD";
       {
-	libspectrum_word wordtemp;
-	wordtemp = readbyte(PC++);
-	wordtemp|= ( readbyte(PC++) << 8 );
-	A=readbyte(wordtemp);
+	z80.memptr.b.l = readbyte(PC++);
+	z80.memptr.b.h = readbyte(PC++);
+	A=readbyte(z80.memptr.w++);
       }
 LD
         } elsif( $src eq '(REGISTER+dd)' ) {
@@ -765,7 +784,8 @@ LD
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); PC++;
-	$dest = readbyte( REGISTER + (libspectrum_signed_byte)offset );
+	z80.memptr.w = REGISTER + (libspectrum_signed_byte)offset;
+	$dest = readbyte( z80.memptr.w );
       }
 LD
         }
@@ -801,6 +821,12 @@ LD
 	my $register = substr $dest, 1, 2;
 
 	if( length $src == 1 ) {
+	    if( $register eq 'BC' or $register eq 'DE' ) {
+	        print << "LD";
+      z80.memptr.b.l=$register+1;
+      z80.memptr.b.h=A;
+LD
+            }
 	    print << "LD";
       writebyte($register,$src);
 LD
@@ -817,6 +843,8 @@ LD
       {
 	libspectrum_word wordtemp = readbyte( PC++ );
 	wordtemp|=readbyte(PC++) << 8;
+	z80.memptr.b.l = wordtemp + 1;
+	z80.memptr.b.h = A;
 	writebyte(wordtemp,A);
       }
 LD
@@ -835,24 +863,26 @@ LD
     } elsif( $dest eq '(REGISTER+dd)' ) {
 
 	if( length $src == 1 ) {
-	print << "LD";
+	  print << "LD";
       {
 	libspectrum_byte offset;
 	offset = readbyte( PC );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 );
 	contend_read_no_mreq( PC, 1 ); PC++;
-	writebyte( REGISTER + (libspectrum_signed_byte)offset, $src );
+	z80.memptr.w = REGISTER + (libspectrum_signed_byte)offset;
+	writebyte( z80.memptr.w, $src );
       }
 LD
         } elsif( $src eq 'nn' ) {
-	    print << "LD";
+	  print << "LD";
       {
 	libspectrum_byte offset, value;
 	offset = readbyte( PC++ );
 	value = readbyte( PC );
 	contend_read_no_mreq( PC, 1 ); contend_read_no_mreq( PC, 1 ); PC++;
-	writebyte( REGISTER + (libspectrum_signed_byte)offset, value );
+	z80.memptr.w = REGISTER + (libspectrum_signed_byte)offset;
+	writebyte( z80.memptr.w, value );
       }
 LD
         }
@@ -892,9 +922,11 @@ sub opcode_OUT (@) {
 
     if( $port eq '(nn)' and $register eq 'A' ) {
 	print << "OUT";
-      { 
-	libspectrum_word outtemp;
-	outtemp = readbyte( PC++ ) + ( A << 8 );
+      {
+	libspectrum_byte nn = readbyte( PC++ );
+	libspectrum_word outtemp = nn | ( A << 8 );
+	z80.memptr.b.h = A;
+	z80.memptr.b.l = (nn + 1);
 	writeport( outtemp, A );
       }
 OUT
@@ -904,6 +936,7 @@ OUT
 	} else {
 	    print "      writeport( BC, $register );\n";
 	}
+	print "      z80.memptr.w = BC + 1;\n";
     }
 }
 
@@ -990,6 +1023,7 @@ sub opcode_RLD (@) {
 	writebyte(HL, (bytetemp << 4 ) | ( A & 0x0f ) );
 	A = ( A & 0xf0 ) | ( bytetemp >> 4 );
 	F = ( F & FLAG_C ) | sz53p_table[A];
+	z80.memptr.w=HL+1;
       }
 RLD
 }
@@ -1026,6 +1060,7 @@ sub opcode_RRD (@) {
 	writebyte(HL,  ( A << 4 ) | ( bytetemp >> 4 ) );
 	A = ( A & 0xf0 ) | ( bytetemp & 0x0f );
 	F = ( F & FLAG_C ) | sz53p_table[A];
+	z80.memptr.w=HL+1;
       }
 RRD
 }
@@ -1075,9 +1110,9 @@ sub opcode_shift (@) {
 
 	print << "shift";
       {
-	libspectrum_word tempaddr; libspectrum_byte opcode3;
+	libspectrum_byte opcode3;
 	contend_read( PC, 3 );
-	tempaddr =
+	z80.memptr.w =
 	    REGISTER + (libspectrum_signed_byte)readbyte_internal( PC );
 	PC++; contend_read( PC, 3 );
 	opcode3 = readbyte_internal( PC );
@@ -1087,7 +1122,7 @@ sub opcode_shift (@) {
 #include "z80_ddfdcb.c"
 	}
 #else			/* #ifdef HAVE_ENOUGH_MEMORY */
-	z80_ddfdcbxx(opcode3,tempaddr);
+	z80_ddfdcbxx(opcode3);
 #endif			/* #ifdef HAVE_ENOUGH_MEMORY */
       }
 shift
@@ -1194,18 +1229,18 @@ while(<>) {
 	    my $hexmask = res_set_hexmask( $opcode, $bit );
 
 	    print << "CODE";
-      $register = readbyte(tempaddr) $operator $hexmask;
-      contend_read_no_mreq( tempaddr, 1 );
-      writebyte(tempaddr, $register);
+      $register = readbyte(z80.memptr.w) $operator $hexmask;
+      contend_read_no_mreq( z80.memptr.w, 1 );
+      writebyte(z80.memptr.w, $register);
       break;
 CODE
 	} else {
 
 	    print << "CODE";
-      $register=readbyte(tempaddr);
-      contend_read_no_mreq( tempaddr, 1 );
+      $register=readbyte(z80.memptr.w);
+      contend_read_no_mreq( z80.memptr.w, 1 );
       $opcode($register);
-      writebyte(tempaddr, $register);
+      writebyte(z80.memptr.w, $register);
       break;
 CODE
 	}
